package com.fjhx.email.config;

import cn.hutool.core.thread.ThreadUtil;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableAsync;

import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * 线程池配置
 */
@EnableAsync
@Configuration
public class TaskPoolConfig {

    public static final String emailInfoTaskExecutor = "emailInfoTaskExecutor";
    public static final String emailContentTaskExecutor = "emailContentTaskExecutor";

    /**
     * 获取邮件基本信息线程池
     */
    @Bean(name = emailInfoTaskExecutor)
    public ThreadPoolExecutor emailInfoTaskExecutor() {
        return new ThreadPoolExecutor(
                10,
                20,
                60,
                TimeUnit.SECONDS,
                new ArrayBlockingQueue<>(30),
                (runnable, executor) -> {
                    // 等待5秒之后重新入线程池
                    ThreadUtil.sleep(3000);
                    executor.execute(runnable);
                }
        );
    }

    /**
     * 获取邮件正文、附件、地址线程池
     */
    @Bean(name = emailContentTaskExecutor)
    public ThreadPoolExecutor emailContentTaskExecutor() {
        return new ThreadPoolExecutor(
                3,
                20,
                60,
                TimeUnit.SECONDS,
                new ArrayBlockingQueue<>(20),
                (runnable, executor) -> {
                    // 本线程执行
                    runnable.run();
                }
        );
    }

}
